from django.test import TestCase
from .models import FlatPage
from django.contrib.auth.models import User
# Create your tests here.
class views_Test(TestCase):
    def test_registration_required(self):
        # create page
        page=FlatPage.objects.create(url="/test/",title='testing page',
                                     content='testing content',
                                     meta='testing meta',
                                     registration_required=True)
        page.sites.add(1)
        # get response
        response = self.client.get(page.get_absolute_url())

        # test
        self.assertEqual(response.status_code,302)

    def logged_in_user_can_access_registration_required_page(self):
        # create page
        page = FlatPage.objects.create(url="/test/", title='testing page',
                                       content='testing content',
                                       meta='testing meta',
                                       registration_required=True)
        page.sites.add(1)

        # login user
        user=User.objects.create_user('testuser',password='testpassword')
        self.client.login(username='testuser',password='testpassword')

        # get response
        response = self.client.get(page.get_absolute_url())

        # test
        self.assertEqual(response.status_code, 200)

    def test_registration_not_required(self):
        # create page
        page = FlatPage.objects.create(url="/test/", title='testing page',
                                       content='testing content',
                                       meta='testing meta',
                                       registration_required=False)
        page.sites.add(1)
        # get response
        response = self.client.get(page.get_absolute_url())

        # test
        self.assertEqual(response.status_code, 200)

    def test_try_to_render_custom_template(self):
        # create page
        page = FlatPage.objects.create(url="/test/", title='testing page',
                                       content='testing content',
                                       meta='testing meta',
                                       registration_required=False,template_name='betterFlatPages/contact_page.html')
        page.sites.add(1)
        # get response
        response = self.client.get(page.get_absolute_url())

        self.assertIn('betterFlatPages/base.html',(t.name for t in response.templates))

class views_api_Test(TestCase):
    def test_registration_required(self):
        # create page
        page=FlatPage.objects.create(url="/test/",title='testing page',
                                     content='testing content',
                                     meta='testing meta',
                                     registration_required=True)
        page.sites.add(1)
        # get response
        response = self.client.get('/api/pages/test/')

        # test
        self.assertEqual(response.status_code,403)

    def logged_in_user_can_access_registration_required_page(self):
        # create page
        page = FlatPage.objects.create(url="/test/", title='testing page',
                                       content='testing content',
                                       meta='testing meta',
                                       registration_required=True)
        page.sites.add(1)

        # login user
        user=User.objects.create_user('testuser',password='testpassword')
        self.client.login(username='testuser',password='testpassword')

        # get response
        response = self.client.get('/api/pages/test/')

        # test
        self.assertEqual(response.status_code, 200)

    def test_registration_not_required(self):
        # create page
        page = FlatPage.objects.create(url="/test/", title='testing page',
                                       content='testing content',
                                       meta='testing meta',
                                       registration_required=False)
        page.sites.add(1)
        # get response
        response = self.client.get('/api/pages/test/')

        # test
        self.assertEqual(response.status_code, 200)




from django.contrib import admin
from .models import FlatPage
from .forms import FlatpageForm
from django.utils.translation import gettext_lazy as _
# Register your models here.
@admin.register(FlatPage)
class FlatPageAdmin(admin.ModelAdmin):
    form = FlatpageForm
    fieldsets = (
        (None, {'fields': ('url', 'title', 'content', 'sites')}),
        (_('Advanced options'), {
            'classes': ('collapse',),
            'fields': ('registration_required','enable_comments', 'template_name','meta'),
        }),
    )
    list_display = ('url', 'title')
    list_filter = ('sites', 'registration_required')
    search_fields = ('url', 'title')



from django.contrib.sites.shortcuts import get_current_site
from rest_framework.generics import get_object_or_404
from rest_framework import serializers
from rest_framework.response import Response
from rest_framework.status import HTTP_403_FORBIDDEN
from rest_framework.views import APIView

from betterFlatPages.models import FlatPage

class flatPageSerializer(serializers.ModelSerializer):
    class Meta:
        model = FlatPage

        exclude = ['sites','registration_required','template_name','id']


class flatPageAPIView(APIView):
    serializer_class = flatPageSerializer

    # override default permissions
    permission_classes = []

    def get_object(self):
        # get the flat page object
        url = "/" + self.kwargs['url']

        site_id = get_current_site(self.request).id
        return get_object_or_404(FlatPage, url=url, sites=site_id)

    def get(self,request,url):
        flatPage = self.get_object()

        if flatPage.registration_required and not request.user.is_authenticated:
            return Response({"detail": "Authentication credentials were not provided."},status=HTTP_403_FORBIDDEN)

        else:
            return Response(self.serializer_class(flatPage).data)


from django.http import HttpResponse
from django.shortcuts import render,get_object_or_404
from django.contrib.sites.shortcuts import get_current_site
from .models import FlatPage
from django.contrib.auth.views import redirect_to_login
from django.template import loader

DEFAULT_TEMPLATE = 'betterFlatPages\default.html'

# Create your views here.
def flatPageView(request,url):
    # get the flat page object
    url="/"+url
    site_id = get_current_site(request).id
    flatPage = get_object_or_404(FlatPage,url=url,sites=site_id)

    if flatPage.registration_required and not request.user.is_authenticated:
        return redirect_to_login(request.path)

    # get the flat page template
    if flatPage.template_name:
        template = loader.select_template((flatPage.template_name, DEFAULT_TEMPLATE))
    else:
        template = loader.get_template(DEFAULT_TEMPLATE)

    return HttpResponse(template.render({'flatpage': flatPage}, request))
